@@ -30,6 +30,7 @@ class QuantEngineClient:
         self.ws = None
         self.CorIdCounter = FastReadCounter()
         self.TickQueue = BoundedBlockingQueue(1000000)
+        self.HandlerMessageQueue = BoundedBlockingQueue(1000000)
         self.stop_event = threading.Event()
         self.open_event = threading.Event()
         self.BacktestStartDate: datetime = datetime.utcnow()
@@ -39,6 +40,7 @@ class QuantEngineClient:
         self.ProgressPercent: float = 0.0
         self.Thread1 = None
         self.Thread2 = None
+        self.Thread3 = None
 
         self.add_message_handler("tick", self.tick_handler)
         self.add_message_handler("order_update", self.order_update_handler)
@@ -75,7 +77,7 @@ class QuantEngineClient:
             "set_data",
             AlgorumMessageType.Request,
             self.CorIdCounter.increment(),
-            quant_data,
+            jsonpickle.encode(quant_data, False),
             None
         ))
 
@@ -286,6 +288,19 @@ class QuantEngineClient:
             else:
                 break
 
+    def handle_message(self):
+
+        while 1:
+            algorum_websocket_message = self.HandlerMessageQueue.dequeue()
+
+            if algorum_websocket_message.Name in self.MessageHandlerMap:
+                handler = self.MessageHandlerMap[algorum_websocket_message.Name]
+            else:
+                handler = None
+
+            if handler is not None:
+                handler(algorum_websocket_message)
+
     def initialize(self):
         def on_message(ws, message):
             # diagnostic message
@@ -313,7 +328,7 @@ class QuantEngineClient:
                             del self.CorIdMessageMap[msg.CorId]
                             async_waiter.WaiterEvent.set()
             else:
-                handler(msg)
+                self.HandlerMessageQueue.enqueue(msg)
 
         def on_error(ws, error):
             print(error)
@@ -337,5 +352,7 @@ class QuantEngineClient:
         self.Thread1.start()
         self.Thread2 = threading.Thread(target=self.process_tick, daemon=True)
         self.Thread2.start()
+        self.Thread3 = threading.Thread(target=self.handle_message, daemon=True)
+        self.Thread3.start()
 
         self.open_event.wait()
